(define-module (ui nagios-secondary ajax)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :export (on-load))

(define *typelist*
  `(("service-alerts" "alerts" ,(_ "Alerts"))))

(define *services* '())

(define (update-services)
  (catch/message
   (lambda ()
     (set! *services* (append (woo-list "/nagios-secondary/services")
                              (woo-list "/nagios-secondary/secondary"))))))

(define *is-new* #f)

(define (new-check)
  (set! *is-new* #t)
  (form-update-activity "params" #t)
  (form-update-value "secondary" "")
  (update-services)
  (form-update-enum "servname" *services*)
  (if (and (string-null? (form-value "servname"))
           (not (null? *services*)))
    (form-update-value "servname" (woo-get-option (car *services*) 'name)))
  (servname-checktype-changed))

(define (object-name item)
  (car (string-tokenize item)))

(define (object-desc item)
  (let ((tokens (string-tokenize item)))
    (if (> (length tokens) 1)
      (let ((desc (string-join (cdr tokens))))
        (if (and (= (string-index desc #\() 0)
                 (= (string-index desc #\))
                    (- (string-length desc) 1)))
          (substring desc 1 (- (string-length desc) 1))
          desc))
      "")))

(define (find-service name)
  (filter (lambda (s)
            (equal? name (woo-get-option s 'name)))
          *services*))

(define (service-desc servname)
  (let ((service (find-service servname)))
    (if (not (null? service))
      (object-desc (woo-get-option (car service) 'label))
      "")))

(define (autoupdate-name-and-desc)
  (let* ((type (form-value "checktype"))
         (typeitem (filter (lambda (ti)
                             (equal? type (car ti)))
                           *typelist*))
         (name (string-split (form-value "name") #\-))
         (desc (string-tokenize (form-value "desc")))
         (servname (form-value "servname"))
         (servdesc (service-desc servname)))
    (if (and (not (string-null? servname))
             (not (null? typeitem)))
      (cond
       ((string-null? (car name))
        (form-update-value "name"
                           (string-append servname "-"
                                          (cadar typeitem))))
       ((and (> (length name) 1)
             (member (car (reverse name))
                     (map cadr *typelist*)))
        (if (member (string-join (reverse (cdr (reverse name))) "-")
                    (map (lambda (s) (woo-get-option s 'name))
                         *services*))
          (form-update-value "name"
                             (string-join (list servname (cadar typeitem))
                                          "-"))
          (form-update-value "name"
                             (string-join (append (reverse (cdr (reverse name)))
                                                  (list (cadar typeitem)))
                                          "-"))))))
    (if (and (not (null? typeitem))
             (not (string-null? servdesc)))
      (cond
       ((null? desc)
        (form-update-value "desc"
                           (string-append servdesc " "
                                          (caddar typeitem))))
       ((and (> (length desc) 1)
             (member (car (reverse desc))
                     (map caddr *typelist*)))
        (if (member (string-join (reverse (cdr (reverse desc))))
                    (map (lambda (s) (object-desc (woo-get-option s 'label)))
                         *services*))
        (form-update-value "desc"
                           (string-join (list servdesc (caddar typeitem))
                                        " "))
        (form-update-value "desc"
                           (string-join (append (reverse (cdr (reverse desc)))
                                                  (list (caddar typeitem)))
                                        " "))))))))

(define (servname-checktype-changed)
  (if *is-new* (autoupdate-name-and-desc))
  (for-each
   (lambda (group)
     (form-update-visibility group
                             (equal? group
                                     (string-append (form-value "checktype")
                                                    "-opts"))))
   '("service-alerts-opts")))

(define (save-check)
  (update-services)
  (let ((name (form-value "name")))
    (if (or (not *is-new*)
            (null? (find-service name))
            (form-confirm
             (_ "Another check with the same name exists. Overwrite?")))
      (begin
        (catch/message
         (lambda ()
           (woo-write (string-append "/nagios-secondary/secondary/" name)
                      'checktype (form-value "checktype")
                      'desc (form-value "desc")
                      'hosts (form-value "hosts")
                      'hostgroups (form-value "hostgroups")
                      'servname (form-value "servname")
                      'state (form-value "state")
                      'interval (form-value "interval")
                      'wtimes (form-value "wtimes")
                      'ctimes (form-value "ctimes"))))
        (update-secondary name)))))

(define (update-secondary . args)
  (catch/message
   (lambda ()
     (form-update-enum "secondary"
                       (woo-list "/nagios-secondary/secondary"))))
  (if (not (null? args))
    (begin
      (form-update-value "secondary" (car args))
      (read-secondary (car args)))))

(define (update-lists)
  (catch/message
   (lambda ()
     (form-update-enum "hosts"
                       (woo-list "/nagios-secondary/hosts"))
     (form-update-enum "hostgroups"
                       (woo-list "/nagios-secondary/hostgroups"))
     (form-update-enum "checktype"
                       (woo-list "/nagios-secondary/checktypes"))
     (form-update-enum "state"
                       (woo-list "/nagios-secondary/states"))))
  (update-services)
  (form-update-enum "servname" *services*))

(define (read-secondary . args)
  (let ((name (or (and (not (null? args))
                       (car args))
                  (form-value "secondary"))))
    (if (and name (not (string-null? name)))
      (begin
        (catch/message
         (lambda ()
           (let ((data (woo-read-first
                        (string-append "/nagios-secondary/secondary/" name)
                        'language (form-value "language"))))
             (form-update-value-list '("checktype"
                                       "name"
                                       "desc"
                                       "hosts"
                                       "hostgroups"
                                       "servname"
                                       "state"
                                       "interval"
                                       "wtimes"
                                       "ctimes") data))))
        (form-update-activity "params" #t)
        (servname-checktype-changed)
        (set! *is-new* #f))
      (form-update-activity "params" #f))))

(define (delete-check)
  (let ((name (form-value "name")))
    (if (or *is-new*
            (form-confirm
             (_ "Delete this check?")))
      (begin
        (catch/message
         (lambda ()
           (woo "delete"
                (string-append "/nagios-secondary/secondary/" name))))
        (update-secondary)))))

(define (on-load)
  (form-update-activity "params" #f)
  (form-bind "new" "click" new-check)
  (form-bind "apply" "click" save-check)
  (form-bind "reset" "click" read-secondary)
  (form-bind "delete" "click" delete-check)
  (form-bind "checktype" "change" servname-checktype-changed)
  (form-bind "servname" "change" servname-checktype-changed)
  (form-bind "secondary" "change" read-secondary)
  (update-lists)
  (update-secondary))
